/*
 * Copyright (C) CERN 2016
 * Author: Federico Vaga <federico.vaga@cern.ch>
 * License: GPLv3
 */

#include <mockturtle-rt-smem.h>
#include <mockturtle-framework.h>

static unsigned int iteration;
static uint32_t period_c;
static unsigned int period = 1000000;
static unsigned int alarm_enable;
static uint32_t alarm_iter;


enum rt_slot_name {
	AC_CMD_IN = 0,
	AC_CMD_OUT,
	AC_DATA,
};

static struct rt_mq mq[] = {
	[AC_CMD_IN] = {
		.type = TRTL_HMQ,
		.index = 0,
		.flags = TRTL_RT_MQ_FLAGS_INPUT,
	},
	[AC_CMD_OUT] = {
		.type = TRTL_HMQ,
		.index = 0,
		.flags = TRTL_RT_MQ_FLAGS_OUTPUT,
	},
	[AC_DATA] = {
		.type = TRTL_HMQ,
		.index = 1,
		.flags = TRTL_RT_MQ_FLAGS_OUTPUT | TRTL_RT_MQ_FLAGS_CLAIM,
	},
};

enum ac_variable {
	AC_TIME = 0,
	AC_PERIOD_UPDATE,
	AC_ALARM_EN,
	AC_ALARM_ITER,
};

static struct rt_variable variables[] = {
	[AC_TIME] = {
		.addr = (void *)&iteration,
		.mask = 0xFFFFFFFF,
		.offset = 0,
		.flags = 0,
	},
	[AC_PERIOD_UPDATE] = {
		.addr = (void *)&period,
		.mask = 0xFFFFFFFF,
		.offset = 0,
		.flags = 0,
	},
	[AC_ALARM_EN] = {
		.addr = (void *)&alarm_enable,
		.mask = 0xFFFFFFFF,
		.offset = 0,
		.flags = 0,
	},
	[AC_ALARM_ITER] = {
		.addr = (void *)&alarm_iter,
		.mask = 0xFFFFFFFF,
		.offset = 0,
		.flags = 0,
	},
};

static struct rt_application app = {
	.name = "alarm-clk",
	.version = {
		.fpga_id = CONFIG_FPGA_APPLICATION_ID,
		.rt_id = CONFIG_RT_APPLICATION_ID,
		.rt_version = RT_VERSION(0, 1),
		.git_version = GIT_VERSION
	},
	.mq = mq,
	.n_mq = ARRAY_SIZE(mq),

	.variables = variables,
	.n_variables = ARRAY_SIZE(variables),
};


static void ac_update(void)
{
	uint32_t sec, cyc;

	rt_get_time(&sec, &cyc);

	if ((--period_c) == 0) {
		period_c = period;
		trtl_rt_mq_send_uint32(AC_DATA, 0x12, 1,
				       iteration);
		pr_debug("Iteration %d\n\r", iteration);
	}

	if (alarm_enable) {
		if (alarm_iter < iteration) {
			trtl_rt_mq_send_uint32(AC_DATA, 0x34, 2,
					       iteration, alarm_iter);
			alarm_enable = 0;
			alarm_iter = 0;
		}
	}

	iteration++;
}


/**
 * Firmware initialization
 */
static void ac_init(void)
{
	period_c = period;
}


/**
 * Well, the main :)
 */
int main()
{
	rt_init(&app);
	ac_init();

	while (1) {
		/*
		 * Handle all messages incoming from slot
		 * AC_CMD_IN
		 * as actions
		 */
		rt_mq_action_dispatch(AC_CMD_IN);

		ac_update();
	}

	return 0;
}
