/*
 * Copyright (C) CERN 2016
 * Author: Federico Vaga <federico.vaga@cern.ch>
 * License: GPLv3
 */

#include <mockturtle-framework.h>

static uint32_t period_c;
static unsigned int period = 1000000;

#define DG_BUF_SIZE (32)
static uint32_t dg_data[DG_BUF_SIZE];


struct dg_conf {
	uint32_t gain;
	uint32_t offset;
};
static struct dg_conf dg_conf;

static uint32_t dg_last_sample_idx;
static uint32_t dg_last_sample = 1;


enum rt_slot_name {
	DG_MQ_CMD_IN = 0,
	DG_MQ_CMD_OUT,
	DG_MQ_DATA,
};

static struct rt_mq mq[] = {
	[DG_MQ_CMD_IN] = {
		.type = TRTL_HMQ,
		.index = 0,
		.flags = TRTL_RT_MQ_FLAGS_INPUT,
	},
	[DG_MQ_CMD_OUT] = {
		.type = TRTL_HMQ,
		.index = 0,
		.flags = TRTL_RT_MQ_FLAGS_OUTPUT,
	},
	[DG_MQ_DATA] = {
		.type = TRTL_HMQ,
		.index = 1,
		.flags = TRTL_RT_MQ_FLAGS_OUTPUT | TRTL_RT_MQ_FLAGS_CLAIM,
	},
};

enum dg_variable {
	DG_PERIOD_UPDATE = 0,
};

static struct rt_variable variables[] = {
	[DG_PERIOD_UPDATE] = {
		.addr = (void *)&period,
		.mask = 0xFFFFFFFF,
		.offset = 0,
		.flags = 0,
	},
};

enum dg_structures {
	DG_DATA = 0,
	DG_CONF,
};

struct rt_buffer buffers[] = {
	[DG_DATA] = {
		.buf = dg_data,
		.len = sizeof(dg_data),
	},
	[DG_CONF] = {
		.buf = &dg_conf,
		.len = sizeof(struct dg_conf)
	},
};

static struct rt_application app = {
	.name = "data-gen",
	.version = {
		.fpga_id = CONFIG_FPGA_APPLICATION_ID,
		.rt_id = CONFIG_RT_APPLICATION_ID,
		.rt_version = RT_VERSION(0, 1),
		.git_version = GIT_VERSION
	},
	.mq = mq,
	.n_mq = ARRAY_SIZE(mq),

	.variables = variables,
	.n_variables = ARRAY_SIZE(variables),

	.buffers = buffers,
	.n_buffers = ARRAY_SIZE(buffers),
};

static void generate_sample(void)
{
	if (dg_last_sample == 0)
		dg_last_sample = 1;

	dg_last_sample++;
	if (dg_last_sample * dg_conf.gain < dg_last_sample)
		dg_last_sample = 1;
	dg_last_sample = (dg_last_sample * dg_conf.gain) + dg_conf.offset;


	pr_debug("%s:%d [%"PRIu32"d/%d] = %"PRIu32"u\n\r", __func__, __LINE__,
		 dg_last_sample_idx + 1, DG_BUF_SIZE, dg_last_sample);

	dg_data[dg_last_sample_idx] = dg_last_sample;
	dg_last_sample_idx = ++dg_last_sample_idx & (DG_BUF_SIZE - 1);
}

static void dg_update(void)
{
	if ((--period_c) == 0) {
		period_c = period;
		generate_sample();
	}
}


/**
 * Firmware initialization
 */
static void dg_init(void)
{
	period_c = period;
	dg_last_sample_idx = 0;
	dg_last_sample = 1;
	dg_conf.offset = 0;
	dg_conf.gain = 1;
}


/**
 * Well, the main :)
 */
int main()
{
	rt_init(&app);
	dg_init();

	while (1) {
		/*
		 * Handle all messages incoming from slot
		 * DG_MQ_CMD_IN
		 * as actions
		 */
		rt_mq_action_dispatch(DG_MQ_CMD_IN);

		dg_update();
	}

	return 0;
}
