/*
 * Copyright (C) 2013-2014 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * Released according to the GNU GPL, version 2 or any later version.
 */

#include <string.h>
#include <mockturtle-rt.h>
#include <fw-svec-common.h>
#include <mockturtle-framework.h>

#define GPIO_CODR	0x0 /* Clear Data Register */
#define GPIO_SODR	0x4 /* Set Data Register */
#define GPIO_DDR	0x8 /* Direction Data Register */
#define GPIO_PSR	0xC /* Status Register */

enum rt_slot_name {
	SVEC_CMD_IN = 0,
	SVEC_CMD_OUT,
};

struct rt_mq mq[] = {
	[SVEC_CMD_IN] = {
		.index = 0,
		.flags = TRTL_RT_MQ_FLAGS_HMQ | TRTL_RT_MQ_FLAGS_INPUT,
	},
	[SVEC_CMD_OUT] = {
		.index = 0,
		.flags = TRTL_RT_MQ_FLAGS_HMQ | TRTL_RT_MQ_FLAGS_OUTPUT,
	},
};


static struct svec_structure svec_struct;

struct rt_buffer svec_buffers[] = {
	[SVEC_BUF_TEST] = {
		.buf = &svec_struct,
		.len = sizeof(struct svec_structure),
	}
};

struct rt_variable svec_variables[] = {
	[SVEC_VAR_LEMO_STA] = {
		.addr = TRTL_ADDR_DP(GPIO_PSR),
		.mask = PIN_LEMO_MASK,
		.offset = 0,
	},
	[SVEC_VAR_LEMO_DIR] = {
		.addr = TRTL_ADDR_DP(GPIO_DDR),
		.mask = PIN_LEMO_MASK,
		.offset = 0,
	},
	[SVEC_VAR_LEMO_SET] = {
		.addr = TRTL_ADDR_DP(GPIO_SODR),
		.mask = PIN_LEMO_MASK,
		.offset = 0,
	},
	[SVEC_VAR_LEMO_CLR] = {
		.addr = TRTL_ADDR_DP(GPIO_CODR),
		.mask = PIN_LEMO_MASK,
		.offset = 0,
	},
	[SVEC_VAR_LED_STA] = {
		.addr = TRTL_ADDR_DP(GPIO_PSR),
		.mask = PIN_LED_MASK,
		.offset = PIN_LED_OFFSET,
	},
	[SVEC_VAR_LED_SET] = {
		.addr = TRTL_ADDR_DP(GPIO_SODR),
		.mask = PIN_LED_MASK,
		.offset = PIN_LED_OFFSET,
	},
	[SVEC_VAR_LED_CLR] = {
		.addr = TRTL_ADDR_DP(GPIO_CODR),
		.mask = PIN_LED_MASK,
		.offset = PIN_LED_OFFSET,
	},
	[SVEC_VAR_AUTO] = {
		.addr = &autosvec_run,
		.mask = 0xFFFFFFFF,
		.offset = 0,
	},
};


struct rt_application app = {
	.name = "svec-svec",
	.version = {
		.fpga_id = FPGA_APPLICATION_ID,
		.rt_id = RT_APPLICATION_ID,
		.rt_version = RT_VERSION(1, 0),
		.git_version = GIT_VERSION
	},
	.mq = mq,
	.n_mq = 2,

	.buffers = svec_buffers,
	.n_buffers = ARRAY_SIZE(svec_buffers),

	.variables = svec_variables,
	.n_variables = ARRAY_SIZE(svec_variables),
};

/**
 * It sends messages over the debug interface
 */
static void svec_debug_interface(void)
{
	pp_printf("Hello world.\n\r");
	pp_printf("We are messages over the serial interface.\n\r");
	pp_printf("Print here your messages.\n\r");
}


/**
 * Well, the main :)
 */
int main()
{
	rt_init(&app);

	svec_debug_interface();
	while (1) {
		/* Handle all messages incoming from slot SVEC_HMQ_IN
		   as actions */
		rt_mq_action_dispatch(SVEC_CMD_IN);
	}

	return 0;
}
