--------------------------------------------------------------------------------
-- CERN BE-CO-HT
-- Mock Turtle
-- https://gitlab.cern.ch/coht/mockturtle
--------------------------------------------------------------------------------
--
-- unit name:   mt_per_cpu_csr_pkg
--
-- description: Per CPU CSR ports, not to be confused with the wbgen generated
-- file that describes the whole CSR interface.
--
--------------------------------------------------------------------------------
-- Copyright CERN 2018
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package mt_per_cpu_csr_pkg is

  -- Per CPU CSR inputs (from urv to mt)
  type t_mt_per_cpu_csr_in is record
    host_int_i       : std_logic;
    udata_i          : std_logic_vector(31 downto 0);
    dbg_cpu_status_i : std_logic;
    dbg_insn_ready_i : std_logic;
    dbg_mbx_i        : std_logic_vector(31 downto 0);
  end record t_mt_per_cpu_csr_in;

  type t_mt_per_cb_csr_in is record
    cpu_i            : t_mt_per_cpu_csr_in;
    host_int_i       : std_logic;
    hmqi_status_i    : std_logic_vector(7 downto 0);
    hmqo_status_i    : std_logic_vector(7 downto 0);
  end record t_mt_per_cb_csr_in;

  -- Per CPU CSR outputs (from mt to urv)
  type t_mt_per_cpu_csr_out is record
    reset_o         : std_logic;
    uaddr_addr_o    : std_logic_vector(19 downto 0);
    core_sel_o      : std_logic;
    udata_o         : std_logic_vector(31 downto 0);
    udata_load_o    : std_logic;
    dbg_cpu_force_o : std_logic;
    dbg_insn_o      : std_logic_vector(31 downto 0);
    dbg_insn_wr_o   : std_logic;
    dbg_mbx_o       : std_logic_vector(31 downto 0);
    dbg_mbx_wr_o    : std_logic;
  end record t_mt_per_cpu_csr_out;

  type t_mt_per_cb_csr_out is record
    cpu_o            : t_mt_per_cpu_csr_out;
    host_int_o      : std_logic;
  end record t_mt_per_cb_csr_out;

end package mt_per_cpu_csr_pkg;
