//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   MockTurtleDriver
//
// description: A SystemVerilog Class to provide an abstraction of the complete
// MockTurtle core.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __MOCK_TURTLE_DRIVER_INCLUDED
 `define __MOCK_TURTLE_DRIVER_INCLUDED

 `include "simdrv_defs.svh"
 `include "mt_cpu_csr_driver.svh"
 `include "mt_mqueue_host_driver.svh"
 `include "mt_config_rom_driver.svh"

class MockTurtleDriver;
   protected CBusAccessor m_acc;
   protected uint64_t m_base;
   MTCPUControl m_csr;
   MQueueHost m_hmq;
   MTConfigRom m_rom;

   function  new ( CBusAccessor acc, uint64_t base );
      m_acc = acc;
      m_base = base;
   endfunction // new

   task init();
      m_csr = new ( m_acc, m_base + 'hc000 );
      m_hmq = new ( m_acc, m_base + 'h0000 );
      m_rom = new ( m_acc, m_base + 'he000, 256 );
      m_csr.init();
      m_hmq.init();
      m_rom.init();
   endtask // init

   task load_firmware ( int core, string filename, bit check = 1'b1);
      m_csr.load_firmware ( core, filename, check );
   endtask // load_firmware

   task reset_core ( int core, int reset );
      m_csr.reset_core ( core, reset );
   endtask // reset_core

   task cfg_rom_display();
      m_rom.pretty_print();
   endtask // cfg_rom_display

   // polls HMQ and Debug UART
   task update();
      m_csr.update();
      m_hmq.update();
   endtask // update

endclass // MockTurtleDriver

`endif //  `ifndef __MOCK_TURTLE_DRIVER_INCLUDED
