//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   MTConfigRom
//
// description: A ROM to hold the various configuration values of the current
// MockTurtle implementation.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __MOCK_TURTLE_CONFIG_ROM_INCLUDED
 `define __MOCK_TURTLE_CONFIG_ROM_INCLUDED

 `include "simdrv_defs.svh"

class MTConfigRom;
   protected CBusAccessor m_acc;
   protected uint64_t     m_base;
   protected uint32_t     m_size;
   protected uint32_t     m_data [];

   function new ( CBusAccessor acc, uint64_t base, uint64_t size );
      m_acc  = acc;
      m_base = base;
      m_size = size;
      m_data = new[m_size];
   endfunction // new

   task _readl (  uint32_t r, ref uint32_t v );
      uint64_t tmp;
      m_acc.read ( m_base + r, tmp );
      v= tmp;
   endtask // _readl

   task init ( );
      int i;

      for ( i = 0; i < m_size; i+=4 )
	begin
	   _readl ( i, m_data[i] );
	end
   endtask // init

   function uint32_t read ( uint32_t pos );
      return m_data[4 * pos];
   endfunction // read

   task pretty_print ( );
      $display ( "Configuration ROM contents:" );
      $display ( "  Magic Value: %s",  read(0) );
      $display ( "  MT revision: %0d", read(1) );

   endtask // pretty_print

endclass // MockTurtleConfigRom

`endif // `ifndef __MOCK_TURTLE_CONFIG_ROM_DRIVER_INCLUDED
