//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   main
//
// description: A SystemVerilog testbench to exercise all the main features of
// Mock Turtle core.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`include "mock_turtle_driver.svh"
`include "vhd_wishbone_master.svh"

`timescale 1ns/1ps

module main;

   reg rst_n = 0;
   reg clk_sys = 0;

   always #8ns clk_sys <= ~clk_sys;

   initial begin
      repeat(20) @(posedge clk_sys);
      rst_n = 1;
   end

   wire host_irq;

   IVHDWishboneMaster Host ( clk_sys, rst_n );

   mock_turtle_core #
     (
      .g_double_core_clock(1'b0)
      )
   DUT (
	.clk_i   (clk_sys),
	.rst_n_i (rst_n),
	.host_slave_i (Host.master.out),
	.host_slave_o (Host.master.in),
	.host_irq_o(host_irq)
	);

   initial begin
      automatic CWishboneAccessor acc;
      automatic MockTurtleDriver drv;

      #10us;

      acc = Host.get_accessor();
      acc.set_mode (PIPELINED);

      drv = new(acc, 'h20000);
      drv.init();

      drv.load_firmware(0, "../sw/hello/hello.ram.urv");
      drv.reset_core(0, 0);

      forever begin
	 drv.update();
	 #1us;
      end

   end // initial begin

endmodule
