/*
 * This work is part of the White Rabbit Node Core project.
 *
 * Copyright (C) 2013-2014 CERN (www.cern.ch)
 * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 *
 * Released according to the GNU GPL, version 2 or any later version.
 */

#include <string.h>
#include <stdint.h>

#include "mockturtle-rt.h"
#include "hw/mockturtle_addresses.h"
#include "hw/mockturtle_queue.h"
#include "hw/mockturtle_cpu_lr.h"

volatile unsigned int counts[8] __attribute__((section(".smem")));

static const int cpu = 0;

static void
failed(char c)
{
  putchar (c);
  puts ("-FAILED!\n");
  counts[cpu] |= 0xff00;
  while (1)
    ;
}

static void
test_notify (void)
{
  unsigned int v;

  puts("#2 notify irq\n");
  v = counts[cpu];
  lr_writel(1, MT_CPU_LR_REG_HOST_INT);
  while ((lr_readl(MT_CPU_LR_REG_HOST_INT) & (1 << cpu)) != 0)
    ;
  if (counts[cpu] != v + 1)
    failed('N');
}

static const char msg[] = "Hello!";

static void
test_hmq(void)
{
  int i;
  unsigned char *p;
  unsigned *hdr;
  unsigned int v;

  puts("#3 hmq\n");
  v = counts[cpu];

  //  Send a message
  mq_claim(TRTL_HMQ, 0);
  p = mq_map_out_buffer(TRTL_HMQ, 0);
  for (i = 0; i < sizeof (msg); i++)
    p[i] = msg[i];
  hdr = mq_map_out_header(TRTL_HMQ, 0);
  *hdr = (sizeof (msg) + 3) / 4;
  mq_send(TRTL_HMQ, 0, 0);

  //  Wait for answer
  while((mq_poll() & 1) == 0)
    ;
  if (counts[cpu] != v + 1)
    failed('H');

  //  Read answer
  hdr = mq_map_in_header(TRTL_HMQ, 0);
  if (*hdr != 2)
    failed('h');
  p = mq_map_in_buffer(TRTL_HMQ, 0);
  for (i = 0; i < sizeof (msg); i++)
    if ((~p[i] & 0xff) != msg[i])
      failed('K');
  mq_discard(TRTL_HMQ, 0);

  if ((mq_poll() & 1) != 0)
    failed('k');
}

static void
test_rmq(void)
{
  int i;
  char *p;
  unsigned *hdr;
  unsigned int v;

  puts("#4 rmq\n");
  v = counts[cpu];

  //  Send a message
  mq_claim(TRTL_RMQ, 0);
  p = mq_map_out_buffer(TRTL_RMQ, 0);
  for (i = 0; i < sizeof (msg); i++)
    p[i] = msg[i];
  hdr = mq_map_out_header(TRTL_RMQ, 0);
  *hdr = (sizeof (msg) + 3) / 4;
  mq_send(TRTL_RMQ, 0, 0);

  //  Wait for answer
  while((mq_poll() & (1 << 16)) == 0)
    ;

  //  Read answer
  hdr = mq_map_in_header(TRTL_RMQ, 0);
  if (*hdr != (sizeof (msg) + 3) / 4)
    failed('R');
  p = mq_map_in_buffer(TRTL_RMQ, 0);
  for (i = 0; i < sizeof (msg); i++)
    if (p[i] != msg[i])
      failed('r');
  mq_discard(TRTL_RMQ, 0);

  if ((mq_poll() & (1 << 16)) != 0)
    failed('s');
}

int
main(void)
{
  counts[cpu] = 0;

  puts("#1 console\n");

  test_notify();

  test_rmq();

  test_hmq();

  puts ("Done\n");

  return 0;
}
