/**
 * @defgroup framework-action Framework Action
 * @{
 * @ingroup framework
 * @brief Action (RPC)
 * @copyright (C) 2015-2016 CERN (www.cern.ch)
 * @author Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: MPL-2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#ifndef __TRTL_FW_FRM_ACTION_H__
#define __TRTL_FW_FRM_ACTION_H__

/**
 * Action prototype type
 * @param[in] hin input message header
 * @param[in] pin input message payload
 * @param[out] hout output message header
 * @param[out] pout output message payload
 * @return 0 on success. -1 on error
 */
typedef int (action_t)(struct trtl_proto_header *hin, void *pin,
		       struct trtl_proto_header *hout, void *pout);

/**
 * This is an @ref action_t function type. It fills the output payload
 * with an acknowledged message
 */
static inline void rt_send_ack(struct trtl_proto_header *hin, void *pin,
			       struct trtl_proto_header *hout, void *pout)
{
	hout->msg_id = TRTL_MSG_ID_ACK;
	hout->len = 0;
}

/**
 * This is an @ref action_t function type. It fills the output payload
 * with a not-acknowledged message
 */
static inline void rt_send_nack(struct trtl_proto_header *hin, void *pin,
				struct trtl_proto_header *hout, void *pout)
{
	hout->msg_id = TRTL_MSG_ID_NACK;
	hout->len = 0;
}

extern int rt_recv_ping(struct trtl_proto_header *hin, void *pin,
			struct trtl_proto_header *hout, void *pout);
extern int rt_version_getter(struct trtl_proto_header *hin, void *pin,
			     struct trtl_proto_header *hout, void *pout);
extern int rt_variable_setter(struct trtl_proto_header *hin, void *pin,
			    struct trtl_proto_header *hout, void *pout);
extern int rt_variable_getter(struct trtl_proto_header *hin, void *pin,
			    struct trtl_proto_header *hout, void *pout);
extern int rt_buffer_setter(struct trtl_proto_header *hin, void *pin,
			       struct trtl_proto_header *hout, void *pout);
extern int rt_buffer_getter(struct trtl_proto_header *hin, void *pin,
			       struct trtl_proto_header *hout, void *pout);
extern int rt_mq_action_dispatch(unsigned int mq_in);

#endif
/**@}*/
