/**
 * @defgroup framework-common Framework Common Utilities
 * @{
 * @ingroup framework
 * @brief Common Utilities
 * @copyright (C) 2015-2016 CERN (www.cern.ch)
 * @author Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: MPL-2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#ifndef __TRTL_FW_FRM_COMMON_H__
#define __TRTL_FW_FRM_COMMON_H__

#include "mockturtle-frm-action.h"
#include "mockturtle-frm-mqueue.h"
#include "mockturtle-frm-variable.h"
#include "mockturtle-frm-buffer.h"

#define RT_VERSION_MAJ(_v) ((_v >> 16) & 0xFFFF)
#define RT_VERSION_MIN(_v) (_v & 0xFFFF)
#define RT_VERSION(_a, _b) (((_a & 0xFFFF) << 16) | (_b & 0xFFFF))


/**
 * Real-Time Application Descriptor
 */
struct rt_application {
	const char name[16]; /**< Firmware name*/
	const uint32_t *fpga_id_compat; /**< list of compatible FPGA
					   application ID */
	const unsigned int fpga_id_compat_n; /**< number of entry in
						the fpga_id_compat list */
	const struct trtl_rt_version version; /**< version running */

	struct rt_mq *mq; /**< list of used MQ */
	uint8_t n_mq; /**< number of available MQ */

	struct rt_buffer *buffers; /**< exported buffers */
	unsigned int n_buffers; /**< number or exported buffers */

	struct rt_variable *variables; /**< exported variables */
	unsigned int n_variables; /**< number or exported variables */

	action_t **actions; /**< list of custum actions */
	unsigned int n_actions; /**< number of custum actions */
};


extern struct rt_application *_app;

extern int rt_init(struct rt_application *app);
extern void rt_get_time(uint32_t *seconds, uint32_t *cycles);
extern void rt_memcpy(uint32_t *dest, uint32_t *src, size_t n);

#endif
/**@}*/
