/**
 * Copyright (C) 2016 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: MPL-2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include <mockturtle-framework.h>

/**
 * The function can be used to send debug information to the host
 * trought the HMQ
 * @param[in] mq_in MQ index within mq declaration in rt_application
 * @param[in] n_values number of variadic arguments
 * @return 0 on success. -1 on error
 */
int rt_send_debug(int mq_in, int n_values, ...)
{
	va_list ap;
	struct trtl_msg out_buf;
	uint32_t *buf;
	struct trtl_proto_header hdr = {
		.rt_app_id = _app->version.rt_id,
		.msg_id = TRTL_MSG_ID_DBG,
		.slot_io = _app->mq[mq_in].index & 0xF,
		.seq = 0,
		.len = n_values,
		.flags = 0x0,
		.trans = 0x0,
		.time = 0x0,
	};
	int i;

	if (!HAS_MOCKTURTLE_FRAMEWORK_DEBUG_ENABLE) {
		return 0;
	}

	out_buf = rt_mq_claim_out(&hdr);
	buf = (uint32_t *)rt_proto_payload_get(out_buf.data);

	va_start(ap, n_values);
	for (i = 0; i < hdr.len;++i)
		buf[i] = va_arg(ap, uint32_t);
	va_end(ap);

	rt_proto_header_set((void *) out_buf.data, &hdr);
	rt_mq_msg_send(&out_buf);

	return 0;
}

/**
 * It prints on the serial interface the given message header
 * @param[in] h message header to print
 */
void rt_print_header(struct trtl_proto_header *h)
{
	if (!HAS_MOCKTURTLE_FRAMEWORK_DEBUG_ENABLE) {
		return;
	}

	pr_debug("    app_id 0x%x | msg_id %d | slot_io 0x%x | seq %"PRIu32"d\n\r",
		  h->rt_app_id, h->msg_id, h->slot_io, h->seq);
	delay(1000);
	pr_debug("    len %d | flags 0x%x | trans 0x%x | time %"PRIu32"d\n\r",
		  h->len, h->flags, h->trans, h->time);
	delay(1000);
}

/**
 * It prints on the serial interface a buffer
 * @param[in] d buffer data
 * @param[in] count number of 32bit words to show
 */
void rt_print_data(uint32_t *d, unsigned int count)
{
	int i;

	if (!HAS_MOCKTURTLE_FRAMEWORK_DEBUG_ENABLE) {
		return;
	}

	for (i = 0; i < count; i++) {
		pr_debug("%s: data[%d] = 0x%"PRIx32"\n\r", __func__, i , d[i]);
		delay(1000);
	}
}
