/**
 * @defgroup framework-mqueue Framework Message Queue
 * @{
 * @ingroup framework
 * @brief Message Queues definitions and functions
 * @copyright (C) 2015-2016 CERN (www.cern.ch)
 * @author Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: MPL-2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#ifndef __TRTL_FW_FRM_MQUEUE_H__
#define __TRTL_FW_FRM_MQUEUE_H__

extern uint32_t msg_seq;

/**
 * Description of the message queue
 */
struct rt_mq {
	enum trtl_mq_type type; /**< MQ type */
	uint8_t index; /**< MQ gateware index */
	unsigned long flags; /**< MQ flag info */
	void *buf; /**< MQ buffer */
};

/**
 * MQ Flag. It is RMQ
 */
#define TRTL_RT_MQ_FLAGS_RMQ (1 << 0)

/**
 * MQ Flag. It is HMQ
 */
#define TRTL_RT_MQ_FLAGS_HMQ (0)

/**
 * MQ Flag. It is output
 */
#define TRTL_RT_MQ_FLAGS_OUTPUT (1 << 1)

/**
 * MQ Flags. On output keep claimed.
 * This means that as soon as you send a message on a slot,
 * the framework will claim the slot immediately.
 */
#define TRTL_RT_MQ_FLAGS_CLAIM (1 << 2)

/**
 * MQ Flag. It is input
 */
#define TRTL_RT_MQ_FLAGS_INPUT (0)


/**
 * It send the message associate to the given header
 * @param[in] msg message to send
 */
static inline void rt_mq_msg_send(struct trtl_msg *msg)
{
	struct trtl_proto_header *hdr;

	hdr = rt_proto_header_get((void *)msg->data);

	/* When we are not using sync messages, we use the global
	   sequence number */
	if (!(hdr->flags & TRTL_HEAD_FLAG_SYNC))
		hdr->seq = msg_seq++;

	mq_send(!!(hdr->flags & TRTL_HEAD_FLAG_REMOTE),
		(hdr->slot_io & 0xF),
		hdr->len + (sizeof(struct trtl_proto_header) / 4));
}


extern void trtl_rt_mq_send(unsigned int mq_idx, unsigned int count);
extern void trtl_rt_mq_send_uint32(unsigned int mq_idx, uint8_t msg_id,
				   unsigned int n, ...);
extern void trtl_rt_mq_send_buf(unsigned int mq_idx, uint8_t msg_id,
				unsigned int n, void *data);
#endif
/**@}*/
