/*
 * Basic printf based on vprintf based on vsprintf
 *
 * Alessandro Rubini for CERN, 2011 -- public domain
 * (please note that the vsprintf is not public domain but GPL)
 */

/**
 * Include this code only if explicitly asked to
 */

#include <stdarg.h>
#include <generated/autoconf.h>

#ifdef CONFIG_MOCKTURTLE_LIBRARY_PRINT_ENABLE

#include <mockturtle-rt-common.h>
#include <mockturtle-rt-serial.h>


static char print_buf[CONFIG_PRINT_BUFSIZE];

int pp_vprintf(const char *fmt, va_list args)
{
	int ret;

	ret = pp_vsprintf(print_buf, fmt, args);
	puts(print_buf);
	return ret;
}

int pp_sprintf(char *s, const char *fmt, ...)
{
	va_list args;
	int ret;

	va_start(args, fmt);
	ret = pp_vsprintf(s, fmt, args);
	va_end(args);
	return ret;
}


int pp_printf(const char *fmt, ...)
{
	va_list args;
	int ret;

	va_start(args, fmt);
	ret = pp_vprintf(fmt, args);
	va_end(args);

	return ret;
}

#ifdef CONFIG_MOCKTURTLE_LIBRARY_PRINT_ERROR_ENABLE
int pr_error(const char *fmt, ...)
{
	va_list args;
	int ret;

	va_start(args, fmt);
	ret = pp_vprintf(fmt, args);
	va_end(args);

	delay(10000);

	return ret;
}
#endif

#ifdef CONFIG_MOCKTURTLE_LIBRARY_PRINT_DEBUG_ENABLE
int pr_debug(const char *fmt, ...)
{
	va_list args;
	int ret;

	va_start(args, fmt);
	ret = pp_vprintf(fmt, args);
	va_end(args);

	delay(10000);

	return ret;
}
#endif

#endif
