import os
import pytest
import PyMockTurtle
import time

@pytest.fixture
def firmware_file_config():
    testdir = os.path.dirname(os.path.abspath(os.path.realpath(__file__)))
    return os.path.join(testdir, "firmware/config_rom/fw-config-rom.bin")


class TestConfig(object):
    def test_exist(self, devid):
        assert os.path.exists("/sys/class/mockturtle/trtl-{:04x}/config-rom".format(devid)) == True

    def test_valid_host(self, trtl_device, cfg):
        assert trtl_device.rom.signature == cfg.signature
        assert trtl_device.rom.app_id == cfg.app_id
        assert trtl_device.rom.n_cpu == cfg.n_cpu

    def test_valid_softcpu(self, trtl_device, trtl_cpu, devid, cfg):
        trtl_hmq = PyMockTurtle.TrtlHmq(devid, 0, PyMockTurtle.TrtlHmq.FLAGS_DIR_OUT)

        trtl_cpu.disable()
        trtl_cpu.load_application_file(firmware_file_config())
        trtl_cpu.enable()

        # Give time to the soft-CPU to build and send the message
        time.sleep(0.1)

        msg = trtl_hmq.recv_msg()

        del trtl_hmq

        assert msg.payload[0] == cfg.signature
        assert msg.payload[5] == cfg.app_id
        assert msg.payload[6] == cfg.n_cpu
