import hashlib
import os
import pytest
import time

@pytest.fixture
def firmware_file_loop():
    testdir = os.path.dirname(os.path.abspath(os.path.realpath(__file__)))
    return os.path.join(testdir, "firmware/cpu-loop/fw-loop.bin")


class TestCPU(object):
    def test_cpu(self):
        pass

    def test_exist(self, trtl_cpu):
        path = "/dev/mockturtle/trtl-{:04x}-cpu-{:02d}".format(trtl_cpu.device_id,
                                                               trtl_cpu.cpu_index)
        assert True == os.path.exists(path)

    def test_disable(self, trtl_cpu):
        trtl_cpu.disable()
        assert False == trtl_cpu.is_enable()

    def test_enable(self, trtl_cpu):
        trtl_cpu.enable()
        assert True == trtl_cpu.is_enable()

    def test_load(self, trtl_cpu):
        trtl_cpu.disable()
        trtl_cpu.load_application_file(firmware_file_loop())
        trtl_cpu.enable()

        time.sleep(1)

        trtl_cpu.disable()
        trtl_cpu.dump_application_file("/tmp/dump.bin")

        with open(firmware_file_loop(), 'rb') as f:
            data_o = f.read()
            md5_o = hashlib.md5(data_o).hexdigest()

        with open("/tmp/dump.bin", 'rb') as f:
            data_d = f.read(len(data_o))
            md5_d = hashlib.md5(data_d).hexdigest()

        assert md5_o == md5_d
